/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2019                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef FLATTREE_H
#define FLATTREE_H

#include "Tree.h"

namespace cubegui
{
class TreeItem;
class FlatTreeData;

class FlatTree : public Tree
{
public:
    FlatTree();
    virtual QString
    getLabel();

protected:
    virtual TreeItem*
    createTree();

    virtual QString
    getItemName( cube::Vertex* vertex );

    /**
     * Assign inclusive and exclusive values to tree items recursively.
     * @param item  Root tree item
     * @param inclusive_values  Vector of inclusive values indexed by Id.
     * @param exclusive_values  Vector of exclusive values indexed by Id.
     * @param differenc_values  Vector of difference values indexed by Id.
     * @note Default implementation has to be overridden to assign difference values to subroutines
     */
    void
    assignValues( TreeItem*                          item,
                  const std::vector< cube::Value* >& inclusive_values,
                  const std::vector< cube::Value* >& exclusive_values,
                  const std::vector< cube::Value* >& difference_values );

    /**
     * Set the basic inclusive and exclusive values in the full tree.
     * @param left  List of trees to the left of this tree
     * @param right List of trees to the right of this tree
     */
    virtual QVector<Task*>
    setBasicValues( const QList<Tree*>&     leftTrees,
                    const QList<Tree*>&     rightTrees,
                    const QList<TreeItem*>& itemsToCalculate );

private:

    void
    computeValue( cube::list_of_metrics& mv,
                  TreeItem*              item );
    void
    computeValue( cube::list_of_metrics&      mv,
                  cube::list_of_sysresources& sv,
                  TreeItem*                   item );

    friend class FlatTreeData;
};


/** for use in QConcurrent */
class FlatTreeData : public Task
{
public:
    FlatTreeData( FlatTree* tree_, const cube::list_of_metrics& m, const cube::list_of_sysresources& s ) :
        tree( tree_ ), metric_selection( m ), sysres_selection( s )
    {
    }

    virtual void
    calculate();

private:
    FlatTree*                        tree;
    const cube::list_of_metrics      metric_selection;
    const cube::list_of_sysresources sysres_selection;
};
}
#endif // FLATTREE_H
