/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2019                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/



#ifndef CUBE_TOOLS_ADVISOR_H
#define CUBE_TOOLS_ADVISOR_H

#include <QPushButton>
#include <QLabel>
#include <QCheckBox>
#include <QListWidget>
#include <QTableWidget>
#include "CubeRatingWidget.h"
#include "CubePlugin.h"
#include "PluginServices.h"
#include "DefaultMarkerLabel.h"

#include "CubeAdvisorToolBar.h"

#include "OverviewPerformanceAnalysis.h"
#include "ComputationPerformanceAnalysis.h"
#include "POPAuditAnalysis.h"
#include "KnlVectorizationAnalysis.h"
#include "KnlMemoryAnalysis.h"
#include "Future.h"

namespace advisor
{
class CubeAdvisor : public QObject, public cubepluginapi::TabInterface, public cubepluginapi::CubePlugin
{
    Q_OBJECT
    Q_INTERFACES( cubepluginapi::CubePlugin )
#if QT_VERSION >= 0x050000
    Q_PLUGIN_METADATA( IID "CubeAdvisorPlugin" )
#endif


public:
    CubeAdvisor();

    ~CubeAdvisor();

    // ContextFreePlugin interface
    virtual QString
    name() const;

    virtual bool
    cubeOpened( cubepluginapi::PluginServices* service );

    virtual void
    cubeClosed();


    virtual void
    version(        int& major,
                    int& minor,
                    int& bugfix ) const;

    virtual QString
    getHelpText() const;

    virtual QString
    getHelpURL() const
    {
        return "Advisor.html";
    }
    // TabInterface implementation
    virtual QWidget*
    widget();

    virtual QString
    label() const;

    QIcon
    icon() const;

    virtual void
    setActive( bool active );



private slots:
    void
    treeItemIsSelected( cubepluginapi::TreeType  type,
                        cubepluginapi::TreeItem* item );

    void
    treasholdChanged( int nv = 0 );


    void
    defineTreeItemMarker();


    void
    tableItemClicked( QTableWidgetItem* );

    void
    showAnalysis( int );

    void
    copyMetrics( bool );

    void
    runAnalysis();

    void
    recalculate( void );

    void
    automatic_recalculate( int );

    void
    initializationFinished( void );

private:
    QWidget* _widget;

    cubepluginapi::PluginServices* service;
    cube::CubeProxy*               cube;
    QStackedWidget*                analyses;
    QComboBox*                     analyses_selector;

    AdvisorToolBar* toolBar;
    int             treashold_value;
    double          total_execution;
    bool            initializationIsFinished;
    bool            startAnalysis;


    QList<const cubepluginapi::TreeItemMarker*> markerList;
    cubepluginapi::DefaultMarkerLabel*          markerLabel;

    cube::Metric*             execution;
    QList<cubegui::TreeItem*> root_callpaths;


    OverviewPerformanceAnalysis*    overview_analysis;
    POPAuditPerformanceAnalysis*    pop_audit_analysis;
    ComputationPerformanceAnalysis* computation_analysis;
    KnlVectorizationAnalysis*       knl_vectorization_analysis;
    KnlMemoryAnalysis*              knl_memory_analysis;
    QPushButton*                    recalculate_button;
    QCheckBox*                      automatic_recalculation;
    QWidget*                        recalculate_widget;

    cubepluginapi::Future* future;
    /*
       CachePerformanceAnalysis *         cache_analysis;
       CPUAnalysis *                      cpu_analysis; */


    void
    createTests();

    void
    calculateOverallTests();

    void
    initialization_run();

    friend class InitialisationTask;
};

class InitialisationTask : public cubepluginapi::Task
{
public:
    InitialisationTask( CubeAdvisor* adv ) : advisor( adv )
    {
    }
    virtual void
    calculate()
    {
        advisor->initialization_run();
    }

protected:
    CubeAdvisor* advisor;
};
}

#endif // CUBE_TOOLS_H
