/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include <config.h>

#include "BranchPredictionPerformanceTest.h"

using namespace advisor;

std::string BranchPredictionPerformanceTest::br_not_measured = QObject::tr(
    "PAPI metrics for branch prediction haven't been measured" ).toStdString();



BranchPredictionPerformanceTest::BranchPredictionPerformanceTest( cube::CubeProxy* cube ) : PerformanceTest( cube )
{
    setName( tr( "Branch prediction success rate" ).toUtf8().data() );
    setWeight( 0.2 );   // need to be adjusted
    branch_prediction = cube->getMetric( "branch_prediction" );
    if ( branch_prediction == NULL )
    {
        setWeight( 0.2 );
        setValue( 0. );
        return;
    }
    cube::list_of_metrics      lmetrics;
    cube::list_of_sysresources lsysres = getRootsOfSystemTree();
    cube::metric_pair          metric;
    metric.first  = branch_prediction;
    metric.second = cube::CUBE_CALCULATE_INCLUSIVE;
    lmetrics.push_back( metric );
    cube->getCallpathTreeValues( lmetrics,
                                 lsysres,
                                 inclusive_values,
                                 exclusive_values );
}

void
BranchPredictionPerformanceTest::applyCnode( const cube::Cnode* cnode, const cube::CalculationFlavour cnf  )
{
    if ( branch_prediction == NULL )
    {
        return;
    }
    double branch_prediction_value = ( cnf == cube::CUBE_CALCULATE_INCLUSIVE ) ?
                                     inclusive_values[ cnode->get_id() ]->getDouble() :
                                     exclusive_values[ cnode->get_id() ]->getDouble();
    setValue( branch_prediction_value );
}

void
BranchPredictionPerformanceTest::applyCnode( const cube::list_of_cnodes& cnodes )
{
    if ( branch_prediction == NULL )
    {
        return;
    }
    if ( cnodes.size() == 1 )
    {
        applyCnode( cnodes[ 0 ].first, cnodes[ 0 ].second );
    }
    else
    {
        cube::value_container      inclusive_values;
        cube::value_container      exclusive_values;
        cube::list_of_sysresources lsysres = getRootsOfSystemTree();
        cube::IdIndexMap           metric_id_indices;
        cube->getMetricSubtreeValues( cnodes,
                                      lsysres,
                                      *branch_prediction,
                                      1,
                                      metric_id_indices,
                                      &inclusive_values,
                                      &exclusive_values );
        double branch_prediction_value = inclusive_values[ 0 ]->getDouble();
        setValue( branch_prediction_value );
    }
}

const std::string&
BranchPredictionPerformanceTest::getCommentText() const
{
    if ( branch_prediction == NULL )
    {
        return br_not_measured;
    }
    else
    {
        return no_comment;
    }
}


// ------ overview tests ---------

bool
BranchPredictionPerformanceTest::isActive() const
{
    return branch_prediction != NULL;
};

bool
BranchPredictionPerformanceTest::isIssue() const
{
    return false;
};
