/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2018                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef CACHE_PERFORMANCE_TEST_H
#define CACHE_PERFORMANCE_TEST_H

#include <string>
#include "PerformanceTest.h"
#include "L1CacheUtilizationPerformanceTest.h"
#include "L2CacheUtilizationPerformanceTest.h"
#include "L3CacheUtilizationPerformanceTest.h"

namespace advisor
{
class CachePerformanceTest : public PerformanceTest
{
private:
    L3CacheUtilizationPerformanceTest* l3_utilization;
    L2CacheUtilizationPerformanceTest* l2_utilization;
    L1CacheUtilizationPerformanceTest* l1_utilization;

    double l3_utilization_weight;
    double l2_utilization_weight;
    double l1_utilization_weight;


protected:

    inline
    void
    applyCnode( const cube::list_of_cnodes& cnodes  )
    {
        ( void )cnodes;
        calculateOverall();
    }

    void
    applyCnode( const cube::Cnode*             cnode,
                const cube::CalculationFlavour cnf = cube::CUBE_CALCULATE_INCLUSIVE )
    {
        ( void )cnode;
        ( void )cnf;
        calculateOverall();
    };


    void
    calculateOverall();

    virtual
    const std::string&
    getCommentText() const;

    inline
    virtual
    const
    QString
    getHelpUrl()
    {
        return ( isActive() ) ?
               QString::fromStdString( "CachePerformanceTest" ) :
               QString::fromStdString( "MissingCachePerformanceTest" );
    }


public:
    CachePerformanceTest( L1CacheUtilizationPerformanceTest*,
                          L2CacheUtilizationPerformanceTest*,
                          L3CacheUtilizationPerformanceTest*
                          );

    virtual
    ~CachePerformanceTest()
    {
    };
// ------ overview tests ---------

    bool
    isActive() const;

    bool
    isIssue() const;
};
};
#endif // ADVISER_RATING_WIDGET_H
