/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2020                                                     **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include <config.h>
#include <cmath>

#include "ComputationLoadBalanceTest.h"

using namespace advisor;


ComputationLoadBalanceTest::ComputationLoadBalanceTest( cube::CubeProxy* cube ) : PerformanceTest( cube )
{
    setName( tr( "Computation Load Balance" ).toUtf8().data() );
    setWeight( 0.2 );   // need to be adjusted
    setValue( 0 );
    comp_norm_imbalance = cube->getMetric( "norm_imbalance_comp" );
    if ( comp_norm_imbalance == NULL )
    {
        return;
    }
    cube::list_of_metrics      lmetrics;
    cube::list_of_sysresources lsysres = getRootsOfSystemTree();
    cube::metric_pair          metric;
    metric.first  = comp_norm_imbalance;
    metric.second = cube::CUBE_CALCULATE_INCLUSIVE;
    lmetrics.push_back( metric );
    cube->getCallpathTreeValues( lmetrics,
                                 lsysres,
                                 inclusive_values,
                                 exclusive_values );
}



void
ComputationLoadBalanceTest::applyCnode( const cube::Cnode* cnode, const cube::CalculationFlavour cnf  )
{
    double n_imbalance = ( cnf == cube::CUBE_CALCULATE_INCLUSIVE ) ?
                         inclusive_values[ cnode->get_id() ]->getDouble() :
                         exclusive_values[ cnode->get_id() ]->getDouble();
    setValue( 1 - pow( n_imbalance, 0.23 ) );
}

void
ComputationLoadBalanceTest::applyCnode( const cube::list_of_cnodes& cnodes )
{
    if ( comp_norm_imbalance == NULL )
    {
        return;
    }
    if ( cnodes.size() == 1 )
    {
        applyCnode( cnodes[ 0 ].first, cnodes[ 0 ].second );
    }
    else
    {
        cube::value_container      inclusive_values;
        cube::value_container      exclusive_values;
        cube::list_of_sysresources lsysres = getRootsOfSystemTree();
        cube::IdIndexMap           metric_id_indices;
        cube->getMetricSubtreeValues( cnodes,
                                      lsysres,
                                      *comp_norm_imbalance,
                                      1,
                                      metric_id_indices,
                                      &inclusive_values,
                                      &exclusive_values );
        double comp_norm_imbalance_value = inclusive_values[ 0 ]->getDouble();
        setValue( 1 - pow( comp_norm_imbalance_value, 0.23 ) );
    }
}

const std::string&
ComputationLoadBalanceTest::getCommentText() const
{
    return no_comment;
}


// ------ overview tests ---------

bool
ComputationLoadBalanceTest::isActive() const
{
    return comp_norm_imbalance != NULL;
};

bool
ComputationLoadBalanceTest::isIssue() const
{
    return false;
};
