/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include <config.h>

#include "OverallManagementPerformanceTest.h"

using namespace advisor;

OverallManagementPerformanceTest::OverallManagementPerformanceTest(
    AlgMgmtTest*                   _alg_mgm,
    CommunicationTest*             _communication,
    GPUOverheadTest*               _gpu_overhead,
    IOOverheadTest*                _io_overhead,
    NonComputationLoadBalanceTest* _non_comp_load_balance
    ) : PerformanceTest( NULL ) // we dont use cube here.. just average results of another tests
{
    alg_mgm               = _alg_mgm;
    communication         = _communication;
    gpu_overhead          = _gpu_overhead;
    io_overhead           = _io_overhead;
    non_comp_load_balance = _non_comp_load_balance;

    setName( tr( "Algorithm Management Overall" ).toUtf8().data() );
    setWeight( 0.5 );   // not important here
}


void
OverallManagementPerformanceTest::calculateOverall()
{
    alg_mgm_weight               = alg_mgm->isActive() ? alg_mgm->weight() : 0.;
    communication_weight         = communication->isActive() ? communication->weight() : 0.;
    gpu_overhead_weight          = gpu_overhead->isActive() ? gpu_overhead->weight() : 0.;
    io_overhead_weight           = io_overhead->isActive()  ? io_overhead->weight() : 0.;
    non_comp_load_balance_weight = non_comp_load_balance->isActive()  ? non_comp_load_balance->weight() : 0.;

    double alg_mgm_value               = alg_mgm->value();
    double communication_value         = communication->value();
    double gpu_overhead_value          = gpu_overhead->value();
    double io_overhead_value           = io_overhead->value();
    double non_comp_load_balance_value = non_comp_load_balance->value();

    double wsum = ( alg_mgm_value * alg_mgm_weight +
                    communication_value * communication_weight +
                    gpu_overhead_value * gpu_overhead_weight +
                    io_overhead_value * io_overhead_weight +
                    non_comp_load_balance_value * non_comp_load_balance_weight
                    );

    double norm = ( alg_mgm_weight +
                    communication_weight +
                    gpu_overhead_weight +
                    non_comp_load_balance_weight
                    );
    setValue( wsum / norm );
}

const std::string&
OverallManagementPerformanceTest::getCommentText() const
{
    return no_comment;
}
