/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include <config.h>

#include "OverallPerformanceTest.h"

using namespace advisor;


OverallPerformanceTest::OverallPerformanceTest(
    OverallComputationPerformanceTest* _computation_overall,
    OverallManagementPerformanceTest*  _management_overall
    ) : PerformanceTest( NULL ) // we dont use cube here.. just average results of another tests
{
    computation_overall = _computation_overall;
    management_overall  = _management_overall;

    setName( tr( "Overall" ).toUtf8().data() );
    setWeight( 1 );   // not important here
}


void
OverallPerformanceTest::calculateOverall()
{
    computation_overall_weight = computation_overall->isActive() ? computation_overall->weight() : 0.;
    management_overall_weight  = management_overall->isActive() ? management_overall->weight() : 0;

    double computation_overall_value = computation_overall->value();
    double management_overall_value  = management_overall->value();

    double wsum = ( computation_overall_value * computation_overall_weight +
                    management_overall_value * management_overall_weight );

    double norm = ( computation_overall_weight +
                    management_overall_weight
                    );
    setValue( wsum / norm );
}

const std::string&
OverallPerformanceTest::getCommentText() const
{
    return no_comment;
}
